# sd_utilities_list.py
# SOLUTIONS version - also fixed line 46 (c.accepted_enrolled, not c.accepted_not_enrolled)
# Sources/people consulted: None
# Apr 27, 2021
# Skeleton by Prof. Lillian Lee (LJL2), Apr 27, 2021

"""
    Utilities for computing showdown info.

    Assumption: inside the directory containing this file, there is a directory
    called "data" that contains college-data files formatted as described in 
    the CS1110 Spring 2021 A3 handout.


    Works with lists of colleges (as opposed to dictionaries as in A3)
   
"""

import college_a5 as college_module
import os

DATADIR="data"


def colleges_from_file(fn):
    """Returns a list of Colleges from file fn, sorted in reverse order of
    number of applications to each college.
    
    Preconditions: 

        fn: nonempty string.  It is the name of a file 

        Inside the directory containing this file, there is a directory
        called "data" that contains college-data files formatted as described in 
        the CS1110 Spring 2021 A3 handout.

        One of the files in that "data" directory has the name `fn`.

    """
    colleges = []
    with open(os.path.join(DATADIR,fn), "r") as fp:
        for studentline in fp:
            process_line(studentline.strip(), colleges)
    colleges.sort(key=lambda c: len(c.rejected) \
                                + len(c.accepted_not_enrolled) \
                                + len(c.accepted_enrolled) + \
                                len(c.waitlisted), 
                  reverse=True)
    return colleges



# STUDENTS: Rewrite the implementation of this function as follows:
#
# Use one or more while-loops to rewrite the "for c in clist" for-loop, 
# where the while-loops don't go through the entire clist unless the name being 
# searched for doesn't show up in any College in clist.
#     
# For the purpose of practicing while-loops, you may NOT call college_named
# (although doing so leads to more compact code), and you may NOT use
# `break`, `continue`, or similar statements.
#
# If you introduce new variables, give them understandable names and/or
# provide comments that explain their intended type and "meaning."
#
# Leave the first 7 lines or so of code alone --- only make changes below
# the "BELOW THIS LINE" comment.

def process_line(sline, clist):
    """ Add the tag of sline to the appropriate list (accepted_enrolled,
    ..., or waitlist) of each College in clist; if sline names a college
    not already represented in clist, a corresponding College is added
    to clist.

    Preconditions: sline is a string formatted as if it were a line from a data file 
    described in the CS1110 Spring 2021 A3 handout. 

    sline does not have trailing newlines.

    clist is a list of Colleges, possibly empty."""

    # STUDENTS: leave the following lines alone; only make changes
    # below where indicated

    tag = int(sline[:sline.index(">>")].strip())

    # name of college student enrolled at
    senrolled_cn = sline[sline.rindex(":")+1:].strip()

    outcomes = sline[sline.index(">>") + len(">>"):sline.rindex('##')].split("##")

    for outcome in outcomes:
        oc = outcome.split(":")
        scname= oc[0].strip() # name of a college supplied by student
        decision = oc[1].strip()

        # STUDENTS: ONLY MAKE CHANGES TO process_line BELOW THIS LINE
        # LEAVE THIS COMMENT IN YOUR SUBMITTED CODE.

    # CODE BEING REPLACED IN THESE SOLUTIONS
    #     found_scname = False # have not found scname in clist yet
    #     found_enrolled = False # have not found enrolled yet (for non-college options)
    #     for c in clist:
    #         if c.name == scname:
    #             found_scname = True
    #             # place tag in correct status list for c
    #             c.add_student(tag, decision, senrolled_cn)
    #             # BEGIN REMOVE
    #             # print("DEBUG: found college, which is now " + str(c))
    #             # END REMOVE
    #         if c.name == senrolled_cn:
    #             found_enrolled = True
    #     if not found_scname:
    #         new_c = college_module.College(scname)
    #         new_c.add_student(tag, decision, senrolled_cn)
    #         clist.append(new_c)
    #         if senrolled_cn == scname:
    #             # now the enrolled school is in clist
    #             found_enrolled = True
    #         # BEGIN REMOVE
    #         #print("DEBUG: created new college " + str(new_c))
    #         # END REMOVE
    # if not found_enrolled:
    #     # apparently "enrolled" in something they didn't apply to, like "gap year"
    #     new_c = college_module.College(senrolled_cn)
    #     new_c.add_student(tag, decision, senrolled_cn)
    #     clist.append(new_c)

    # BEGIN REMOVE
        found_scname = False # have not found scname in clist yet      
        i = 0  # position of College in clist being checked 
        while i < len(clist) and not found_scname:
            c = clist[i]
            if c.name == scname:
                found_scname = True
                c.add_student(tag, decision, senrolled_cn)
            i += 1
        if not found_scname:
            new_c = college_module.College(scname)
            new_c.add_student(tag, decision, senrolled_cn)
            clist.append(new_c)
            if senrolled_cn == scname:
                # now the enrolled school is in clist
                found_enrolled = True 

    # It is better to have this while-loop outside the "for outcome in outcomes",
    # because there's no reason to look for the College the student enrolled at
    # every time you examine one of the Colleges they applied to.
    i = 0
    found_enrolled = False # have not found enrolled yet (for non-college options)
    while i < len(clist) and not found_enrolled:
        c = clist[i]
        if c.name == senrolled_cn:
            found_enrolled = True
        i += 1
    if not found_enrolled:
        # apparently "enrolled" in something they didn't apply to
        # Example: "None;_I_do_not_have_plans_to_attend_college_as_of_now"
        new_c = college_module.College(senrolled_cn)
        new_c.add_student(tag, decision, senrolled_cn)
        clist.append(new_c)

    # END REMOVE

def prompt_for_data(datafilename=""):
    """
    Returns: a list of Colleges from datafile determined by querying the user...
    unless a filename is given, in which case data/filename is the data source.


    Precondition: interaction happens in a directory where a folder named "data"
    is located.
    """
    if len(datafilename) > 0:
        return colleges_from_file(datafilename)

    # A dictionary! Keys are strings because input() returns a string.
    response_map = {
        '1': 'small_test1.txt',
        '2': 'small_cornell_and_suny_test.txt',
        '3': 'a2c_census2020_processed.txt',
        'other': 'some other file in the "data" directory'
    }

    msg = "What college-info file in directory \"data\" should I use?\n"
    for r in ['1', '2', '3', 'other']:
        msg += r + ": " + response_map[r] + "\n"
    msg += ("Default is 1.\n")
    response =input(msg + '\nYour choice? ')
    if response not in response_map:
        # Default to '1'.
        response = '1'

    if response == 'other':
        print("\nOK!. The files I know about are: ")
        for name in glob.glob("data/*"):
            print(name[len("data/"):])
        datafilename = input('Which file you would like? ')
    else:
        datafilename = response_map[response]

    # In contrast to A3, this is not wrapped in try-except, so that students
    # can see what errors are raised by process_line.
    return colleges_from_file(datafilename)


def college_named(n, clist):
    """
    Returns the College in clist that has name `n`, or None (not the string, but 
    the value None) if there is no such College in clist.

    Precondition: clist is a (potentially empty) list of Colleges. None of the
    items in clist is None.  No more than one College in clist can have the
    name `n`.
    """
    for c in clist:
        if c.name == n:
            return c


def menu_listing(colleges):
    """Returns: string of numbered names of the colleges in `colleges`, starting
    with 0.  
    
    Format for each college:  '0: Whatsamatta_U' 
    (so, colon after number, then 1 space.)

    The numbered-name items are separated by a tab (\t).

    Example: for the list colleges_from_file('small_test1.txt'), the output
    is
        '0: B\t1: A\t2: D\t3: E'
    which prints out as
        0: B    1: A    2: D    3: E

    There is no newline ('\n') or whitespace at the end of the returned string.

    Precondition: `colleges` is a list of Colleges (possibly empty.)
    """
    separator = '\t' # in case we later want '\n' instead of '\t' between items
    out = ''
    i = 0
    for c in colleges:
        out += str(i) + ": " + colleges[i].name + separator
        i += 1
    return out.strip()


    # END REMOVE


# BEGIN REMOVE
if __name__ == '__main__':

    print('Running some small checks')

    # colleges = colleges_from_file('small_test1.txt') 
    # colleges = colleges_from_file('small_cornell_and_suny_test.txt')
    colleges = colleges_from_file('a2c_census2020_processed.txt')
    print(menu_listing(colleges))
    #print(college_named('Cornell_U', colleges))

# END REMOVE



